@@ -313,9 +313,9 @@ static int popd_proxy_policy(sasl_conn_t *conn,
 }
 
 static struct sasl_callback mysasl_cb[] = {
-    { SASL_CB_GETOPT, &mysasl_config, NULL },
-    { SASL_CB_PROXY_POLICY, &popd_proxy_policy, (void*) &popd_proxyctx },
-    { SASL_CB_CANON_USER, &popd_canon_user, NULL },
+    { SASL_CB_GETOPT, (mysasl_cb_ft *) &mysasl_config, NULL },
+    { SASL_CB_PROXY_POLICY, (mysasl_cb_ft *) &popd_proxy_policy, (void*) &popd_proxyctx },
+    { SASL_CB_CANON_USER, (mysasl_cb_ft *) &popd_canon_user, NULL },
     { SASL_CB_LIST_END, NULL, NULL }
 };
 
@@ -1006,6 +1006,9 @@ done:
 		if (arg) {
 		    prot_printf(popd_out, "-ERR Unexpected extra argument\r\n");
 		} else {
+		    /* XXX  discard any input pipelined after STLS */
+		    prot_flush(popd_in);
+
 		    cmd_starttls(0);
 		}
 	    }
@@ -1286,6 +1289,7 @@ static void cmd_apop(char *response)
 {
     int sasl_result;
     const void *canon_user;
+    int failedloginpause;
 
     assert(response != NULL);
 
@@ -1307,7 +1311,10 @@ static void cmd_apop(char *response)
 	       popd_clienthost, popd_apop_chal,
 	       sasl_errdetail(popd_saslconn));
 	
-	sleep(3);      
+	failedloginpause = config_getint(IMAPOPT_FAILEDLOGINPAUSE);
+	if (failedloginpause != 0) {
+	    sleep(failedloginpause);
+	}
 
 	/* Don't allow user probing */
 	if (sasl_result == SASL_NOUSER) sasl_result = SASL_BADAUTH;
@@ -1389,6 +1396,8 @@ void cmd_user(char *user)
 
 void cmd_pass(char *pass)
 {
+    int failedloginpause;
+
     if (!popd_userid) {
 	prot_printf(popd_out, "-ERR [AUTH] Must give USER command\r\n");
 	return;
@@ -1438,7 +1447,10 @@ void cmd_pass(char *pass)
 			    strlen(pass))!=SASL_OK) { 
 	syslog(LOG_NOTICE, "badlogin: %s plaintext %s %s",
 	       popd_clienthost, popd_userid, sasl_errdetail(popd_saslconn));
-	sleep(3);
+	failedloginpause = config_getint(IMAPOPT_FAILEDLOGINPAUSE);
+	if (failedloginpause != 0) {
+	    sleep(failedloginpause);
+	}
 	prot_printf(popd_out, "-ERR [AUTH] Invalid login\r\n");
 	free(popd_userid);
 	popd_userid = 0;
@@ -1547,6 +1559,7 @@ void cmd_auth(char *arg)
     char *authtype;
     const void *val;
     const char *canon_user;
+    int failedloginpause;
 
     /* if client didn't specify an argument we give them the list
      *
@@ -1620,7 +1633,10 @@ void cmd_auth(char *arg)
 		       popd_clienthost, authtype);
 	    }
 
-	    sleep(3);
+	    failedloginpause = config_getint(IMAPOPT_FAILEDLOGINPAUSE);
+	    if (failedloginpause != 0) {
+	        sleep(failedloginpause);
+	    }
 
 	    /* Don't allow user probing */
 	    if (sasl_result == SASL_NOUSER) sasl_result = SASL_BADAUTH;
@@ -1782,7 +1798,6 @@ int openinbox(void)
 	popd_login_time = time(0);
 
 	r = mailbox_open_iwl(inboxname, &popd_mailbox);
-	popd_myrights = cyrus_acl_myrights(popd_authstate, popd_mailbox->acl);
 	if (r) {
 	    sleep(3);
 	    syslog(log_level, "Unable to open maildrop %s: %s",
@@ -1792,6 +1807,7 @@ int openinbox(void)
 			error_message(r));
 	    goto fail;
 	}
+	popd_myrights = cyrus_acl_myrights(popd_authstate, popd_mailbox->acl);
 	if (config_popuseacl && !(popd_myrights & ACL_READ)) {
 	    r = (popd_myrights & ACL_LOOKUP) ?
 		 IMAP_PERMISSION_DENIED : IMAP_MAILBOX_NONEXISTENT;
