@@ -235,8 +235,8 @@ int upgrade_index(struct mailbox *mailbox)
 	mailbox->i.options = config_getint(IMAPOPT_MAILBOX_DEFAULT_OPTIONS);
 
     if (oldminor_version < 12) {
-	struct seendata sd = SEENDATA_INITIALIZER;
-	const char *owner_userid;
+	struct seendata sd;
+	char *owner_userid;
 
 	/* remove the CONDSTORE option - it's implicit now */
 	mailbox->i.options &= ~OPT_IMAP_CONDSTORE;
@@ -326,8 +326,7 @@ int upgrade_index(struct mailbox *mailbox)
     }
 no_expunge:
 
-    r = mailbox_repack_setup(mailbox, &repack);
-    if (r) goto fail;
+    mailbox_repack_setup(mailbox, &repack);
 
     /* Write the rest of new index */
     recno = 1;
